@@ -23,7 +23,7 @@ import bisect
 import numpy as np
 
 from pymatgen.core.physical_constants import ELECTRON_CHARGE, EPSILON_0
-from pymatgen.core.structure import Structure
+from pymatgen.util.coord_utils import get_points_in_sphere_pbc
 
 
 class EwaldSummation(object):
@@ -89,20 +89,16 @@ class EwaldSummation(object):
         self._gmax = recip_space_cut if recip_space_cut \
             else 2 * self._sqrt_eta * self._accf
 
-        """
-        The next few lines pre-compute certain quantities and store them. Ewald
-        summation is rather expensive, and these shortcuts are necessary to
-        obtain several factors of improvement in speedup.
-        """
+        # The next few lines pre-compute certain quantities and store them.
+        # Ewald summation is rather expensive, and these shortcuts are
+        # necessary to obtain several factors of improvement in speedup.
         self._oxi_states = [compute_average_oxidation_state(site)
                             for site in structure]
         self._coords = np.array(self._s.cart_coords)
         self._forces = np.zeros((len(structure), 3))
 
-        """
-        Now we call the relevant private methods to calculate the reciprocal
-        and real space terms.
-        """
+        # Now we call the relevant private methods to calculate the reciprocal
+        # and real space terms.
         (self._recip, recip_forces) = self._calc_recip()
         (self._real, self._point, real_point_forces) = \
             self._calc_real_and_point()
@@ -256,12 +252,15 @@ class EwaldSummation(object):
         erecip = np.zeros((numsites, numsites))
         forces = np.zeros((numsites, 3))
         coords = self._coords
-        recip = Structure(self._s.lattice.reciprocal_lattice, ["H"],
-                          [np.array([0, 0, 0])])
-        recip_nn = recip.get_neighbors(recip[0], self._gmax)
+        rcp_latt = self._s.lattice.reciprocal_lattice
+        recip_nn = get_points_in_sphere_pbc(rcp_latt, [[0, 0, 0]], [0, 0, 0],
+                                            self._gmax)
 
-        for (n, dist) in recip_nn:
-            gvect = n.coords
+        frac_to_cart = rcp_latt.get_cartesian_coords
+        for (fcoords, dist, i) in recip_nn:
+            if dist == 0:
+                continue
+            gvect = frac_to_cart(fcoords)
             gsquare = np.linalg.norm(gvect) ** 2
 
             expval = exp(-1.0 * gsquare / (4.0 * self._eta))
@@ -302,7 +301,7 @@ class EwaldSummation(object):
                  - simag * np.cos(gvectdot)) * EwaldSummation.CONV_FACT
             forces += np.tile(factor, (3, 1)).transpose() * gvect_tile
 
-        return (erecip * prefactor * EwaldSummation.CONV_FACT, forces)
+        return erecip * prefactor * EwaldSummation.CONV_FACT, forces
 
     def _calc_real_and_point(self):
         """
@@ -316,7 +315,7 @@ class EwaldSummation(object):
         coords = self._coords
         numsites = self._s.num_sites
         ereal = np.zeros((numsites, numsites))
-        epoint = np.zeros((numsites))
+        epoint = np.zeros(numsites)
         forces = np.zeros((numsites, 3))
         for i in xrange(numsites):
             nn = all_nn[i]  # self._s.get_neighbors(site, self._rmax)
@@ -336,25 +335,25 @@ class EwaldSummation(object):
                 forces[i] += fijpf * (coords[i] - nsite.coords) * \
                     qi * EwaldSummation.CONV_FACT
 
-        ereal = ereal * 0.5 * EwaldSummation.CONV_FACT
-        epoint = epoint * EwaldSummation.CONV_FACT
-        return (ereal, epoint, forces)
+        ereal *= 0.5 * EwaldSummation.CONV_FACT
+        epoint *= EwaldSummation.CONV_FACT
+        return ereal, epoint, forces
 
     @property
     def eta(self):
         return self._eta
 
     def __str__(self):
-        output = ["Real = " + str(self.real_space_energy)]
-        output.append("Reciprocal = " + str(self.reciprocal_space_energy))
-        output.append("Point = " + str(self.point_energy))
-        output.append("Total = " + str(self.total_energy))
-        output.append("Forces:\n" + str(self.forces))
+        output = ["Real = " + str(self.real_space_energy),
+                  "Reciprocal = " + str(self.reciprocal_space_energy),
+                  "Point = " + str(self.point_energy),
+                  "Total = " + str(self.total_energy),
+                  "Forces:\n" + str(self.forces)]
         return "\n".join(output)
 
 
 class EwaldMinimizer:
-    '''
+    """
     This class determines the manipulations that will minimize an ewald matrix,
     given a list of possible manipulations. This class does not perform the
     manipulations on a structure, but will return the list of manipulations
@@ -369,21 +368,21 @@ class EwaldMinimizer:
     order disordered structure transformation.
 
     Author - Will Richards
-    '''
+    """
 
     ALGO_FAST = 0
     ALGO_COMPLETE = 1
     ALGO_BEST_FIRST = 2
 
-    '''
+    """
     ALGO_TIME_LIMIT: Slowly increases the speed (with the cost of decreasing
     accuracy) as the minimizer runs. Attempts to limit the run time to
     approximately 30 minutes.
-    '''
+    """
     ALGO_TIME_LIMIT = 3
 
     def __init__(self, matrix, m_list, num_to_return=1, algo=ALGO_FAST):
-        '''
+        """
         Args:
             matrix:
                 a matrix of the ewald sum interaction energies. This is stored
@@ -401,7 +400,7 @@ class EwaldMinimizer:
                 structures so it may be necessary to overestimate and then
                 remove the duplicates later. (duplicate checking in this
                 process is extremely expensive)
-        '''
+        """
         # Setup and checking of inputs
         self._matrix = copy(matrix)
         # Make the matrix diagonally symmetric (so matrix[i,:] == matrix[:,j])
@@ -441,20 +440,20 @@ class EwaldMinimizer:
         self._minimized_sum = self._output_lists[0][0]
 
     def minimize_matrix(self):
-        '''
+        """
         This method finds and returns the permutations that produce the lowest
         ewald sum calls recursive function to iterate through permutations
-        '''
+        """
         if self._algo == EwaldMinimizer.ALGO_FAST or \
             self._algo == EwaldMinimizer.ALGO_BEST_FIRST:
             return self._recurse(self._matrix, self._m_list,
                                  set(range(len(self._matrix))))
 
     def add_m_list(self, matrix_sum, m_list):
-        '''
+        """
         This adds an m_list to the output_lists and updates the current
         minimum if the list is full.
-        '''
+        """
         if self._output_lists is None:
             self._output_lists = [[matrix_sum, m_list]]
         else:
@@ -468,7 +467,7 @@ class EwaldMinimizer:
             self._current_minimum = self._output_lists[-1][0]
 
     def best_case(self, matrix, m_list, indices_left):
-        '''
+        """
         Computes a best case given a matrix and manipulation list.
 
         Args:
@@ -480,7 +479,7 @@ class EwaldMinimizer:
             indices:
                 Set of indices which haven't had a permutation performed on
                 them.
-        '''
+        """
 
         m_indices = []
         fraction_list = []
@@ -541,7 +540,7 @@ class EwaldMinimizer:
         return next_index
 
     def _recurse(self, matrix, m_list, indices, output_m_list=[]):
-        '''
+        """
         This method recursively finds the minimal permutations using a binary
         tree search strategy.
 
@@ -553,7 +552,7 @@ class EwaldMinimizer:
             indices:
                 Set of indices which haven't had a permutation performed on
                 them.
-        '''
+        """
         #check to see if we've found all the solutions that we need
         if self._finished:
             return
@@ -626,9 +625,9 @@ def compute_average_oxidation_state(site):
     try:
         avg_oxi = sum([sp.oxi_state * occu
                        for sp, occu in site.species_and_occu.items()
-                       if sp != None])
+                       if sp is not None])
         return avg_oxi
-    except:
+    except AttributeError:
         pass
     try:
         return site.charge
