@@ -15,6 +15,8 @@ __date__ = "Sep 23, 2011"
 import re
 import numpy
 import os
+from bz2 import BZ2File
+from gzip import GzipFile
 
 
 def zopen(filename, *args, **kwargs):
@@ -36,11 +38,9 @@ def zopen(filename, *args, **kwargs):
     """
     file_ext = filename.split(".")[-1].upper()
     if file_ext == "BZ2":
-        import bz2
-        return bz2.BZ2File(filename, *args, **kwargs)
+        return BZ2File(filename, *args, **kwargs)
     elif file_ext in ("GZ", "Z"):
-        import gzip
-        return gzip.GzipFile(filename, *args, **kwargs)
+        return GzipFile(filename, *args, **kwargs)
     else:
         return open(filename, *args, **kwargs)
 
@@ -58,9 +58,10 @@ def zpath(filename):
         filename with a zip extension (unless an unzipped version
         exists)
     """
-    for p in [filename, filename + '.gz', filename + '.bz2']:
-        if os.path.exists(p):
-            return p
+    for ext in ["", '.gz', '.GZ', '.bz2', '.BZ2', '.z', '.Z']:
+        zfilename = "{}{}".format(filename, ext)
+        if os.path.exists(zfilename):
+            return zfilename
     return filename
 
 
@@ -197,7 +198,7 @@ def which(program):
     return None
 
 
-def reverse_readline(m_file, blk_size=4096):
+def reverse_readline(m_file, blk_size=4096, max_mem=4000000):
     """
     Generator method to read a file line-by-line, but backwards. This allows
     one to efficiently get data at the end of a file.
@@ -206,11 +207,21 @@ def reverse_readline(m_file, blk_size=4096):
     Raymond Hettinger and Kevin German.
     http://code.activestate.com/recipes/439045-read-a-text-file-backwards-yet-another-implementat/
 
+    Reads file forwards and reverses in memory for files smaller than the
+    max_mem parameter, or for gzip files where reverse seeks are not supported.
+
+    Files larger than max_mem are dynamically read backwards.
+
     Args:
         m_file:
             File stream to read (backwards)
         blk_size:
             The buffer size. Defaults to 4096.
+        max_mem:
+            The maximum amount of memory to involve in this operation. This is
+            used to determine when to reverse a file in-memory versus seeking
+            portions of a file. For bz2 files, this sets the maximum block
+            size.
 
     Returns:
         Generator that returns lines from the file. Similar behavior to the
@@ -218,29 +229,43 @@ def reverse_readline(m_file, blk_size=4096):
         of the file.
     """
 
-    buf = ""
-    m_file.seek(0, 2)
-    lastchar = m_file.read(1)
-    trailing_newline = (lastchar == "\n")
-    
-    while 1:
-        newline_pos = buf.rfind("\n")
-        pos = m_file.tell()
-        if newline_pos != -1:
-            # Found a newline
-            line = buf[newline_pos+1:]
-            buf = buf[:newline_pos]
-            if pos or newline_pos or trailing_newline:
-                line += "\n"
-            yield line
-        elif pos:
-            # Need to fill buffer
-            toread = min(blk_size, pos)
-            m_file.seek(pos-toread, 0)
-            buf = m_file.read(toread) + buf
-            m_file.seek(pos-toread, 0)
-            if pos == toread:
-                buf = "\n" + buf
-        else:
-            # Start-of-file
-            return
+    file_size = os.path.getsize(m_file.name)
+
+    # If the file size is within our desired RAM use, just reverse it in memory
+    # GZip files must use this method because there is no way to negative seek
+    if file_size < max_mem or isinstance(m_file, GzipFile):
+        for line in reversed(m_file.readlines()):
+            yield line.rstrip()
+    else:
+        if isinstance(m_file, BZ2File):
+            # for bz2 files, seeks are expensive. It is therefore in our best
+            # interest to maximize the blk_size within limits of desired RAM
+            # use.
+            blk_size = min(max_mem, file_size)
+
+        buf = ""
+        m_file.seek(0, 2)
+        lastchar = m_file.read(1)
+        trailing_newline = (lastchar == "\n")
+
+        while 1:
+            newline_pos = buf.rfind("\n")
+            pos = m_file.tell()
+            if newline_pos != -1:
+                # Found a newline
+                line = buf[newline_pos+1:]
+                buf = buf[:newline_pos]
+                if pos or newline_pos or trailing_newline:
+                    line += "\n"
+                yield line
+            elif pos:
+                # Need to fill buffer
+                toread = min(blk_size, pos)
+                m_file.seek(pos-toread, 0)
+                buf = m_file.read(toread) + buf
+                m_file.seek(pos-toread, 0)
+                if pos == toread:
+                    buf = "\n" + buf
+            else:
+                # Start-of-file
+                return
